<?php

namespace Drupal\pusher_api\EventSubscriber;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\pusher_api\Event\PusherApiNodeEvent;
use Drupal\pusher_api\PusherApiMainService;
use Pusher\ApiErrorException;
use Pusher\PusherException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Pusher API event subscriber.
 */
class PusherApiNodeSubscriber implements EventSubscriberInterface {

  /**
   * The pusher_api.main service.
   *
   * @var \Drupal\pusher_api\PusherApiMainService
   */
  protected $pusherApi;

  /**
   * The module config.
   *
   * @var \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The Drupal messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a PusherApiNodeSubscriber object.
   *
   * @param \Drupal\pusher_api\PusherApiMainService $pusherApi
   *   The pusher_api.main service.
   */
  public function __construct(PusherApiMainService $pusherApi, ConfigFactory $configFactory, MessengerInterface $messenger) {
    $this->pusherApi = $pusherApi;
    $this->config = $configFactory->get('pusher_api.settings');
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[PusherApiNodeEvent::PUSHER_API_NODE_INSERT][] = 'onNodeInsert';
    $events[PusherApiNodeEvent::PUSHER_API_NODE_UPDATE][] = 'onNodeUpdate';
    $events[PusherApiNodeEvent::PUSHER_API_NODE_DELETE][] = 'onNodeDelete';
    return $events;
  }

  /**
   * Trigger a create event in pusher.
   *
   * @param \Drupal\pusher_api\Event\PusherApiNodeEvent $event
   *   The Pusher API event.
   */
  public function onNodeInsert(PusherApiNodeEvent $event) {
    try {
      $this->doTrigger('create', $event->getEntity());
    }
    catch (ApiErrorException|PusherException $e) {
      $this->messenger->addError($e->getMessage());
    }
  }

  /**
   * Trigger a create event in pusher.
   *
   * @param \Drupal\pusher_api\Event\PusherApiNodeEvent $event
   *   The Pusher API event.
   */
  public function onNodeUpdate(PusherApiNodeEvent $event) {
    try {
      $this->doTrigger('update', $event->getEntity());
    }
    catch (ApiErrorException|PusherException $e) {
      $this->messenger->addError($e->getMessage());
    }
  }

  /**
   * Trigger a create event in pusher.
   *
   * @param \Drupal\pusher_api\Event\PusherApiNodeEvent $event
   *   The Pusher API event.
   */
  public function onNodeDelete(PusherApiNodeEvent $event) {
    try {
      $this->doTrigger('delete', $event->getEntity());
    }
    catch (ApiErrorException|PusherException $e) {
      $this->messenger->addError($e->getMessage());
    }
  }

  /**
   * @param string $event
   *   The event to act on.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @throws \Pusher\ApiErrorException
   * @throws \Pusher\PusherException
   */
  protected function doTrigger($event, $entity) {
    if ($this->config->get('enable_node') === 'yes') {
      $name = 'node_type_' . $entity->getType();
      $configName = $name . '_' . $event;
      if ($this->config->get($configName)) {
        $channel = (!empty($this->config->get($name . '_channel'))) ? $this->config->get($name . '_channel') : FALSE;
        $data = [
          'nid' => $entity->id(),
          'label' => $entity->label(),
          'type' => $entity->getType(),
          'created' => $entity->created->value,
        ];

        // Invoke all the modules that implement one of our hooks.
        $this->invokeDataAlterHooks($entity->getType(), $event, $channel, $entity, $data);

        $this->pusherApi->triggerEvent($event, $data, $channel);
      }
    }
  }

  /**
   * Invoke the hooks developers can implement to alter the data.
   *
   * @param string $type
   *   The node type name,
   * @param string $event
   *   The event name.
   * @param string $channel
   *   The channel name.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param array $data
   *   The data array.
   */
  protected function invokeDataAlterHooks($type, $event, $channel, $entity, array &$data) {
    // Invoke implementations of hook_pusher_api_data_alter.
    $hookName = 'pusher_api_data_alter';
    \Drupal::moduleHandler()->invokeAll($hookName, [$entity, &$data]);

    // Invoke implementations of hook_pusher_api_NODE_TYPE_data_alter.
    $hookName = 'pusher_api_type_' . $type . '_data_alter';
    \Drupal::moduleHandler()->invokeAll($hookName, [$entity, &$data]);

    // Invoke implementations of hook_pusher_api_EVENT_data_alter.
    $hookName = 'pusher_api_event_' . $event . '_data_alter';
    \Drupal::moduleHandler()->invokeAll($hookName, [$entity, &$data]);

    // Invoke implementations of hook_pusher_api_CHANNEL_data_alter.
    $hookName = 'pusher_api_channel_' . $channel . '_data_alter';
    \Drupal::moduleHandler()->invokeAll($hookName, [$entity, &$data]);

    // Invoke implementations of hook_pusher_api_NODE_TYPE_EVENT_data_alter.
    $hookName = 'pusher_api_type_event_' . $type . '_' . $event . '_data_alter';
    \Drupal::moduleHandler()->invokeAll($hookName, [$entity, &$data]);
  }

}
