<?php

namespace Drupal\pusher_api\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Site\Settings;
use Drupal\node\Entity\NodeType;

/**
 * Configure Pusher api settings for this site.
 */
class PusherApiSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pusher_api_pusher_api_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['pusher_api.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = Settings::get('pusher_api');
    $config = $this->config('pusher_api.settings');

    //    /** @var \Drupal\pusher_api\PusherApiMainService $pusher */
    //    $pusher = \Drupal::service('pusher_api.main');
    //    $data['message'] = 'hello world';
    //    $pusher->triggerEvent('test-event', $data);

    if (!isset($settings['api_settings']) || !is_array($settings['api_settings'])) {
      $this->messenger()
        ->addError('Pusher API settings has not been provided in the settings.php.');
    }
    else {
      $form['api_data'] = [
        '#type' => 'details',
        '#title' => $this->t('API data (from settings.php)'),
      ];
      $form['api_data']['pusher_api_app_id'] = [
        '#type' => 'textfield',
        '#default_value' => (!empty($settings['api_settings']['app_id'])) ? $settings['api_settings']['app_id'] : '',
        '#title' => $this->t('App ID'),
        '#disabled' => TRUE,
      ];

      $form['api_data']['pusher_api_key'] = [
        '#type' => 'textfield',
        '#default_value' => (!empty($settings['api_settings']['key'])) ? $settings['api_settings']['key'] : '',
        '#title' => $this->t('App key'),
        '#disabled' => TRUE,
      ];

      $form['api_data']['pusher_api_secret'] = [
        '#type' => 'textfield',
        '#default_value' => $this->obfuscateSecret($settings['api_settings']),
        '#title' => $this->t('App secret'),
        '#disabled' => TRUE,
      ];

      $form['api_data']['pusher_api_cluster'] = [
        '#type' => 'textfield',
        '#default_value' => (!empty($settings['api_settings']['cluster'])) ? $settings['api_settings']['cluster'] : '',
        '#title' => $this->t('Cluster'),
        '#disabled' => TRUE,
      ];

    }

    $form['pusher_api_channel'] = [
      '#type' => 'textfield',
      '#default_value' => (!empty($config->get('pusher_api_channel'))) ? $config->get('pusher_api_channel') : 'pusher_api',
      '#title' => $this->t('Default channel name'),
      '#required' => TRUE,
    ];

    $form['enable_node'] = [
      '#type' => 'select',
      '#title' => $this->t('Send triggers to Pusher for node events'),
      '#default_value' => ($config->get('enable_node')) ? $config->get('enable_node') : 'no',
      '#required' => TRUE,
      '#options' => [
        'yes' => $this->t('Yes'),
        'no' => $this->t('No'),
      ],
    ];

    $nodeTypes = NodeType::loadMultiple();
    $form_state->set('nodeTypes', $nodeTypes);
    if (!empty($nodeTypes)) {
      $form['node_types'] = [
        '#type' => 'details',
        '#title' => $this->t('Node types'),
        '#open' => TRUE,
        '#states' => [
          'visible' => [
            ':input[name="enable_node"]' => ['value' => 'yes'],
          ],
        ],
      ];

      foreach ($nodeTypes as $nodeType) {
        $name = 'node_type_' . $nodeType->id();
        $form['node_types'][$name] = [
          '#type' => 'details',
          '#title' => $nodeType->label(),
        ];

        $form['node_types'][$name][$name . '_create'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Send event on node create'),
          '#default_value' => $config->get($name . '_create'),
        ];

        $form['node_types'][$name][$name . '_update'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Send event on node update'),
          '#default_value' => $config->get($name . '_update'),
        ];

        $form['node_types'][$name][$name . '_delete'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Send event on node delete'),
          '#default_value' => $config->get($name . '_delete'),
        ];

        $form['node_types'][$name][$name . '_channel'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Channel name'),
          '#default_value' => ($config->get($name . '_channel')) ? $config->get($name . '_channel') : $nodeType->id(),
          '#description' => $this->t('When empty the default channel will be used.'),
        ];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Obfuscate the secret.
   *
   * @param array $settings
   *   The settings.
   *
   * @return string
   *   The obfuscated secret, or an empty string.
   */
  protected function obfuscateSecret(array $settings) {
    $secret = '';

    if (!empty($settings['secret'])) {
      $strLength = strlen($settings['secret']);
      $showLength = 3 - $strLength;
      $secret = substr($settings['secret'], 0, $showLength);

      for ($i = 3; $i < $strLength - 3; $i++) {
        $secret .= '*';
      }

      $secret .= substr($settings['secret'], -3);
    }

    return $secret;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('pusher_api.settings')
      ->set('pusher_api_channel', $form_state->getValue('pusher_api_channel'))
      ->set('enable_node', $form_state->getValue('enable_node'));

    if ($form_state->getValue('enable_node') === 'yes') {
      $nodeTypes = $form_state->get('nodeTypes');

      foreach ($nodeTypes as $nodeType) {
        $name = 'node_type_' . $nodeType->id();
        $config->set($name . '_create', $form_state->getValue($name . '_create'))
          ->set($name . '_update', $form_state->getValue($name . '_update'))
          ->set($name . '_delete', $form_state->getValue($name . '_delete'))
          ->set($name . '_channel', $form_state->getValue($name . '_channel'));
      }
    }

    $config->save();
    parent::submitForm($form, $form_state);
  }

}
